@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2013	PgPool Global Development Group
+ * Copyright (c) 2003-2014	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -935,10 +935,17 @@ static void daemonize(void)
 	rc_chdir = chdir("/");
 
 	i = open("/dev/null", O_RDWR);
-	dup2(i, 0);
-	dup2(i, 1);
-	dup2(i, 2);
-
+	if(i < 0)
+	{
+		pool_error("Failed to open \"/dev/null\" reason:%s", strerror(errno));
+	}
+	else
+	{
+		dup2(i, 0);
+		dup2(i, 1);
+		dup2(i, 2);
+		close(i);
+	}
 	/* close syslog connection for daemonizing */
 	if (pool_config->logsyslog) {
 		closelog();
@@ -1572,7 +1579,6 @@ static RETSIGTYPE exit_handler(int sig)
 	{
 		pool_debug("exit_handler: I am not parent");
 		POOL_SETMASK(&UnBlockSig);
-		pool_shmem_exit(0);
 		exit(0);
 	}
 
@@ -1591,6 +1597,11 @@ static RETSIGTYPE exit_handler(int sig)
 
 	exiting = 1;
 
+	/* Close listen socket */
+	pool_log("pgpool main: close listen socket");
+	close(inet_fd);
+	close(unix_fd);
+
 	for (i = 0; i < pool_config->num_init_children; i++)
 	{
 		pid_t pid = process_info[i].pid;
@@ -2708,7 +2719,6 @@ static int find_primary_node(void)
 	BackendInfo *bkinfo;
 	POOL_CONNECTION_POOL_SLOT *s;
 	POOL_CONNECTION *con; 
-	POOL_STATUS status;
 	POOL_SELECT_RESULT *res;
 	bool is_standby;
 	int i;
@@ -2776,25 +2786,31 @@ static int find_primary_node(void)
 			return -1;
 		}
 #endif
-		status = do_query(con, "SELECT pg_is_in_recovery()",
-						  &res, PROTO_MAJOR_V3);
-		if (res->numrows <= 0)
+		if(do_query(con, "SELECT pg_is_in_recovery()", &res, PROTO_MAJOR_V3) == POOL_CONTINUE)
 		{
-			pool_log("find_primary_node: do_query returns no rows");
+			if (res->numrows <= 0)
+			{
+				pool_log("find_primary_node: do_query returns no rows");
+			}
+			if (res->data[0] == NULL)
+			{
+				pool_log("find_primary_node: do_query returns no data");
+			}
+			if (res->nullflags[0] == -1)
+			{
+				pool_log("find_primary_node: do_query returns NULL");
+			}
+			if (res->data[0] && !strcmp(res->data[0], "t"))
+			{
+				is_standby = true;
+			}
 		}
-		if (res->data[0] == NULL)
+		else
 		{
-			pool_log("find_primary_node: do_query returns no data");
+			pool_log("find_primary_node: do_query failed");
 		}
-		if (res->nullflags[0] == -1)
-		{
-			pool_log("find_primary_node: do_query returns NULL");
-		}
-		if (res->data[0] && !strcmp(res->data[0], "t"))
-		{
-			is_standby = true;
-		}   
-		free_select_result(res);
+		if(res)
+			free_select_result(res);
 		discard_persistent_db_connection(s);
 
 		/*
