@@ -711,6 +711,14 @@ pcp_process_info(int pid, int *array_size)
 			}
 			else if (strcmp(buf, "ProcessInfo") == 0)
 			{
+				if(process_info == NULL)
+				{
+					if (debug) fprintf(stderr, "DEBUG: invalid data.\"%s\"\n", buf);
+					free(buf);
+					errorcode = UNKNOWNERR;
+					return NULL;
+				}
+
 				process_info[offset].connection_info = &conn_info[offset];
 
 				index = (char *) memchr(buf, '\0', rsize) + 1;
@@ -723,7 +731,7 @@ pcp_process_info(int pid, int *array_size)
 			
 				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
-					strcpy(process_info[offset].connection_info->user, index);
+					strlcpy(process_info[offset].connection_info->user, index,SM_USER);
 			
 				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
@@ -808,11 +816,22 @@ pcp_systemdb_info(void)
 	}
 	if (debug) fprintf(stderr, "DEBUG: send: tos=\"S\", len=%d\n", ntohl(wsize));
 
-	while (1) {
+	while (1)
+	{
 		if (pcp_read(pc, &tos, 1))
+		{
+			if (buf != NULL)
+				free(buf);
 			return NULL;
+		}
+
 		if (pcp_read(pc, &rsize, sizeof(int)))
+		{
+			if (buf != NULL)
+				free(buf);
 			return NULL;
+		}
+
 		rsize = ntohl(rsize);
 		buf = (char *)malloc(rsize);
 		if (buf == NULL)
@@ -1346,6 +1365,14 @@ pcp_pool_status(int *array_size)
 			}
 			else if (strcmp(buf, "ProcessConfig") == 0)
 			{
+				if(status == NULL)
+				{
+					if (debug) fprintf(stderr, "DEBUG: invalid data.\"%s\"\n", buf);
+					free(buf);
+					errorcode = UNKNOWNERR;
+					return NULL;
+				}
+
 				index = (char *) memchr(buf, '\0', rsize) + 1;
 				if (index != NULL)
 					strlcpy(status[offset].name, index, POOLCONFIG_MAXNAMELEN+1);
