@@ -1244,7 +1244,7 @@ POOL_STATUS Bind(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 {
 	char *pstmt_name;
 	char *portal_name;
-	char *rewrite_msg;
+	char *rewrite_msg = NULL;
 	POOL_SENT_MESSAGE *parse_msg;
 	POOL_SENT_MESSAGE *bind_msg;
 	POOL_SESSION_CONTEXT *session_context;
@@ -1358,6 +1358,8 @@ POOL_STATUS Bind(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 	{
 		return POOL_END;
 	}
+    if(rewrite_msg)
+        free(rewrite_msg);
 
 	return POOL_CONTINUE;
 }
@@ -1533,7 +1535,7 @@ POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend,
 	int i;
 	int len;
 	signed char kind;
-	signed char state;
+	signed char state = 0;
 	POOL_SESSION_CONTEXT *session_context;
 	Node *node = NULL;
 	char *query = NULL;
@@ -2320,6 +2322,7 @@ POOL_STATUS ParameterDescription(POOL_CONNECTION *frontend,
 			if (status < 0)
 			{
 				pool_error("ParameterDescription: error while reading message length");
+                free(p1);
 				return POOL_END;
 			}
 
@@ -2328,8 +2331,10 @@ POOL_STATUS ParameterDescription(POOL_CONNECTION *frontend,
 
 			p = pool_read2(CONNECTION(backend, i), len);
 			if (p == NULL)
+            {
+                free(p1);
 				return POOL_END;
-
+            }
 			if (len != len1)
 			{
 				pool_debug("ParameterDescription: length does not match between backends master(%d) %d th backend(%d) kind:(%c)",
@@ -2356,7 +2361,6 @@ POOL_STATUS ParameterDescription(POOL_CONNECTION *frontend,
 	}
 
 	free(p1);
-
 	return POOL_CONTINUE;
 }
 
@@ -2542,7 +2546,7 @@ POOL_STATUS ProcessFrontendResponse(POOL_CONNECTION *frontend,
 	session_context = pool_get_session_context();
 	if (!session_context)
 	{
-		pool_error("Parse: cannot get session context");
+		pool_error("ProcessFrontendResponse: cannot get session context");
 		return POOL_END;
 	}
 
@@ -2552,7 +2556,7 @@ POOL_STATUS ProcessFrontendResponse(POOL_CONNECTION *frontend,
 	if (pool_read(frontend, &fkind, 1) < 0)
 	{
 		pool_log("ProcessFrontendResponse: failed to read kind from frontend. frontend abnormally exited");
-		return POOL_END;
+		return POOL_END_WITH_FRONTEND_ERROR;
 	}
 
 	pool_debug("ProcessFrontendResponse: kind from frontend %c(%02x)", fkind, fkind);
