@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2011	PgPool Global Development Group
+ * Copyright (c) 2003-2014	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -50,25 +50,42 @@ static char recovery_command[1024];
 
 extern volatile sig_atomic_t pcp_wakeup_request;
 
+/*
+ * Start online recovery.
+ * "recovery_node" is the node to be recovered.
+ * Master or primary node is chosen in this function.
+ */
 int start_recovery(int recovery_node)
 {
 	int node_id;
 	BackendInfo *backend;
 	BackendInfo *recovery_backend;
 	PGconn *conn;
+	int failback_wait_count;
+#define FAILBACK_WAIT_MAX_RETRY 5		/* 5 seconds should be enough for failback operation */
 
 	pool_log("starting recovering node %d", recovery_node);
 
+	if ( (recovery_node < 0) || (recovery_node >= pool_config->backend_desc->num_backends) )
+	{
+		pool_error("start_recovery: node id %d is not valid", recovery_node);
+		return 1;
+	}
+
 	if (VALID_BACKEND(recovery_node))
 	{
+
 		pool_error("start_recovery: backend node %d is alive", recovery_node);
 		return 1;
 	}
 
 	Req_info->kind = NODE_RECOVERY_REQUEST;
 
+	/* select master/primary node */
 	node_id = MASTER_SLAVE ? PRIMARY_NODE_ID : REAL_MASTER_NODE_ID;
 	backend = &pool_config->backend_desc->backend_info[node_id];
+
+	/* get node info to be recovered */
 	recovery_backend = &pool_config->backend_desc->backend_info[recovery_node];
 
 	conn = connect_backend_libpq(backend);
@@ -166,11 +183,19 @@ int start_recovery(int recovery_node)
 	send_failback_request(recovery_node);
 
 	/* wait for failback */
+	failback_wait_count = 0;
 	while (!pcp_wakeup_request)
 	{
 		struct timeval t = {1, 0};
 		/* polling SIGUSR2 signal every 1 sec */
 		select(0, NULL, NULL, NULL, &t);
+		failback_wait_count++;
+		if (failback_wait_count >= FAILBACK_WAIT_MAX_RETRY)
+		{
+			pool_log("start_recovery: waiting for wake up request is timeout(%d seconds)",
+					 FAILBACK_WAIT_MAX_RETRY);
+			break;
+		}
 	}
 	pcp_wakeup_request = 0;
 
@@ -244,6 +269,15 @@ static int exec_recovery(PGconn *conn, BackendInfo *backend, char stage)
 			 backend->backend_data_directory);
 
 	pool_log("starting recovery command: \"%s\"", recovery_command);
+	pool_log("disabling statement_timeout");
+	result = PQexec(conn, "SET statement_timeout To 0");
+	r = (PQresultStatus(result) !=  PGRES_COMMAND_OK);
+	if (r != 0)
+	{
+		pool_error("exec_recovery: SET STATEMENT_TIMEOUT failed at %s",
+				   (stage == FIRST_STAGE) ? "1st stage" : "2nd stage");
+	}
+	PQclear(result);
 
 	pool_debug("exec_recovery: start recovery");
 	result = PQexec(conn, recovery_command);
